C----------------------------------------------------------------------C
C     GX-PC  Program Package for Single Crystal Structure Analysis     C
C     System assembled by L. J. Farrugia,  Department of Chemistry     C
C               Copyright (C) University of Glasgow, 1997              C
C                                                                      C
C     Program      : LAUE                                              C
C     Description  : Examination of Laue symmetry from SHELX .HKL file C
C     Files read   : name.HKL                                          C
C     Files written: none                                              C
C     Author(s)    : L. J. Farrugia - July 1997                        C
C----------------------------------------------------------------------C

      program laue

C ---- ndim is the maximum number of reflections which are read from the
C      .HKL file. This may be increased without problems for UNIX compilers

      parameter(ndim=10000,ndim1=14)
      character filename*12,line*80,laue_classes(ndim1)*10,caption*80
      character*80 spec
      logical lexist
      integer ih(ndim),il(ndim),ik(ndim)
      real f(ndim),sf(ndim),rint(ndim1)
      integer ncomp(ndim1),isum1(ndim1)
      common/laue_c/ih,ik,il,f,sf,nrefl,imerge,ifull
      data laue_classes/'-1','2/m','mmm','4/m','4/mmm','-3(rhomb)',
     1 '-3m(rhomb)','-3(hex)','-3m1(hex)','-31m(hex)','6/m','6/mmm',
     1 'm-3','m-3m'/
      tol=1.0e-6
      sum=0.0
      rtest=0.1
      imerge=2
      ifull=0
      do i=1,ndim
        ih(i)=0
        ik(i)=0
        il(i)=0
        f(i)=0.0
        sf(i)=0.01
      enddo
      nrefl=0
      caption='R(int) is based on F^2'
      open(11,file='laue.lst')
1     write(*,'('' Enter filename (ext .HKL assumed) :'',$)')
      read (*,'(a)') filename
      if(filename .eq. ' ') goto 6
      if(index(filename,'.') .eq. 0) then
        ilen=leng(filename)
        filename(ilen+1:ilen+4)='.hkl'
      endif
      inquire(file=filename,exist=lexist)
      if(.not.lexist) goto 1
10    write(*,'('' Merge on F [1] or on F^2 [2] (default 2):'',$)')
      read(*,'(a)') line
      if(line .eq. ' ') then
        imerge=2
      else
        call trim(line)
        write(spec,102) line(1:leng(line))
102   format('(i',a,')')
        read(line,spec,err=10) imerge
      endif
      if(imerge .ne. 1 .and. imerge .ne. 2) goto 10
      if(imerge .eq. 1) caption='R(int) is based on F'
20    write(*,'('' Write full listing ?: [Y/N]  N : '',$)')
      read(*,'(a)') line
      call trim(line)
      call upcase(line)
      if(line .eq. ' ' .or. line(1:1) .eq. 'N') then
        ifull=0
      elseif(line(1:1) .eq. 'Y') then
        ifull=1
      else
         goto 20
      endif
      open(10,file=filename,status='old',err=4)
      call upcase(filename)

2     read(10,'(a)',end=3,err=4) line
      if(line .eq. ' ') goto 2
      read(line,'(3i4,2f8.2)',err=4) ihh,ikk,ill,ff,sff
      if(ihh .eq. 0 .and. ikk .eq. 0 .and. ill .eq. 0) goto 3
      nrefl=nrefl+1
      if(nrefl .gt. ndim) then
        write(*,'(/'' More than '',i6,'' reflections read. Remainder ign
     1ored'')')ndim
        write(11,'(/'' More than '',i6,'' reflections read. Remainder ig
     1nored'')')ndim
        nrefl=ndim
        goto 3
      endif
      ih(nrefl)=ihh
      ik(nrefl)=ikk
      il(nrefl)=ill
      if(ff .lt. tol) ff= 0.01
      if (abs(sff) .le. tol) sff=0.01
      sum=sum+abs(ff/sff)

      if(imerge .eq. 1) then 
        ff=sqrt(ff)
        sff=0.5*sff/ff
      endif

      f(nrefl)=ff
      sf(nrefl)=abs(sff)
      goto 2
3     continue
C ----all reflections read

      if(nrefl .eq. 0)  goto 5
      write(11,100) nrefl,' reflections read from file',filename,
     1 ' Mean I/sig(I)  = ',(sum/float(nrefl)),caption
      write(*,100) nrefl,' reflections read from file',filename,
     1 ' Mean I/sig(I)  = ',(sum/float(nrefl)),caption
100   format(/i6,a,1x,a,1x,a,f8.2/2x,a)
      write(*,101)
      write(11,101)
101   format(/1x,'Laue class',5x,'R(int)',4x,'N(obs)',4x,'N(ind)')

      do i=1,14
        call laue_sub1(i,rint(i),ncomp(i),isum1(i))
        if(ncomp(i) .eq. 0) then
           write(*,'(2x,a,7x,2(a,9x),a,5x,a)') laue_classes(i), '-',
     1      '-','-',' <<< no equivalents'
           write(11,'(2x,a,7x,2(a,9x),a,5x,a)') laue_classes(i), '-',
     1      '-','-',' <<< no equivalents'
        else
            write(*,'(2x,a,f10.3,2i10,f10.3)') laue_classes(i),rint(i),
     1      ncomp(i),isum1(i)
            write(11,'(2x,a,f10.3,2i10,f10.3)') laue_classes(i),rint(i),
     1      ncomp(i),isum1(i)
        endif
      enddo
      do i=14,1,-1
        if(rint(i) .lt. rtest) goto 7
      enddo
      write(*,'(/''  Cannot determine diffraction symmetry '')')
      write(11,'(/''  Cannot determine diffraction symmetry '')')
      goto 6
7     write(*,'(/''  Highest diffraction symmetry with reasonable R(int)
     1 = '' ,a)') laue_classes(i)
      write(11,'(/''  Highest diffraction symmetry with reasonable R(int
     1) = '' ,a)') laue_classes(i)
      goto 6
4     write(*,'('' Error reading reflection file '',a)') filename
      goto 6
5     write(*,'('' No reflections in file '',a)') filename
6     close(10)
      end
C
C----------------------------------------------------------------------C
C
      subroutine laue_sub1(imode,rint,isum,isum1)
      parameter(ndim=10000)
      integer ih(ndim),il(ndim),ik(ndim),im(3)
      integer mat(3,3),mat1(3,3,15),mat2(3,3,16),mat3(3,3,16),
     1 mat4(3,3,18),mat5(3,3,12)
      integer ihkl(ndim),int(ndim),vm
      real f(ndim),sf(ndim),fsave1(ndim),fsave2(ndim),f_sorted(ndim),
     1 sf_sorted(ndim)
      common/laue_k/kh,kk,korig,k0
      common/laue_c/ih,ik,il,f,sf,nrefl,imerge,ifull
      data mat/1,0,0,0,1,0,0,0,1/
      data mat1/
     1          -1, 0, 0, 0,-1, 0, 0, 0,-1,        !-h -k -l   1
     1          -1, 0, 0, 0, 1, 0, 0, 0,-1,        !-h  k -l   2
     1           1, 0, 0, 0,-1, 0, 0, 0, 1,        ! h -k  l   3
     1          -1, 0, 0, 0,-1, 0, 0, 0, 1,        !-h -k  l   4
     1           1, 0, 0, 0, 1, 0, 0, 0,-1,        ! h  k -l   5
     1           1, 0, 0, 0,-1, 0, 0, 0,-1,        ! h -k -l   6
     1          -1, 0, 0, 0, 1, 0, 0, 0, 1,        !-h  k  l   7
     1           0,-1, 0, 1, 0, 0, 0, 0, 1,        !-k  h  l   8
     1           0, 1, 0,-1, 0, 0, 0, 0,-1,        ! k -h -l   9
     1           0, 1, 0,-1, 0, 0, 0, 0, 1,        ! k -h  l   10
     1           0,-1, 0, 1, 0, 0, 0, 0,-1,        !-k  h -l   11
     1           0, 1, 0, 1, 0, 0, 0, 0,-1,        ! k  h -l   12
     1           0,-1, 0,-1, 0, 0, 0, 0, 1,        !-k -h  l   13
     1           0,-1, 0,-1, 0, 0, 0, 0,-1,        !-k -h -l   14
     1           0, 1, 0, 1, 0, 0, 0, 0, 1/        ! k  h  l   15
      data mat2/ 0, 0, 1, 1, 0, 0, 0, 1, 0,        ! l  h  k   1
     1           0, 0, 1,-1, 0, 0, 0,-1, 0,        ! l -h -k   2
     1           0, 0,-1,-1, 0, 0, 0, 1, 0,        !-l -h  k   3
     1           0, 0,-1, 1, 0, 0, 0,-1, 0,        !-l  h -k   4
     1           0, 0,-1,-1, 0, 0, 0,-1, 0,        !-l -h -k   5
     1           0, 0,-1, 1, 0, 0, 0, 1, 0,        !-l  h  k   6
     1           0, 0, 1, 1, 0, 0, 0,-1, 0,        ! l  h -k   7
     1           0, 0, 1,-1, 0, 0, 0, 1, 0,        ! l -h  k   8
     1           0, 1, 0, 0, 0, 1, 1, 0, 0,        ! k  l  h   9
     1           0,-1, 0, 0, 0, 1,-1, 0, 0,        !-k  l -h   10
     1           0, 1, 0, 0, 0,-1,-1, 0, 0,        ! k -l -h   11
     1           0,-1, 0, 0, 0,-1, 1, 0, 0,        !-k -l  h   12
     1           0,-1, 0, 0, 0,-1,-1, 0, 0,        !-k -l -h   13
     1           0, 1, 0, 0, 0,-1, 1, 0, 0,        ! k -l  h   14
     1           0,-1, 0, 0, 0, 1, 1, 0, 0,        !-k  l  h   15
     1           0, 1, 0, 0, 0, 1,-1, 0, 0/        ! k  l -h   16
      data mat3/ 0, 0,-1, 0, 1, 0, 1, 0, 0,        !-l  k  h   1
     1           0, 0,-1, 0,-1, 0,-1, 0, 0,        !-l -k -h   2
     1           0, 0, 1, 0, 1, 0,-1, 0, 0,        ! l  k -h   3
     1           0, 0, 1, 0,-1, 0, 1, 0, 0,        ! l -k  h   4
     1           0, 0, 1, 0,-1, 0,-1, 0, 0,        ! l -k -h   5
     1           0, 0, 1, 0, 1, 0, 1, 0, 0,        ! l  k  h   6
     1           0, 0,-1, 0,-1, 0, 1, 0, 0,        !-l -k  h   7
     1           0, 0,-1, 0, 1, 0,-1, 0, 0,        !-l  k -h   8
     1           1, 0, 0, 0, 0,-1, 0, 1, 0,        ! h -l  k   9
     1          -1, 0, 0, 0, 0,-1, 0,-1, 0,        !-h -l -k   10
     1          -1, 0, 0, 0, 0, 1, 0, 1, 0,        !-h  l  k   11
     1           1, 0, 0, 0, 0, 1, 0,-1, 0,        ! h  l -k   12
     1          -1, 0, 0, 0, 0, 1, 0,-1, 0,        !-h  l -k   13
     1           1, 0, 0, 0, 0, 1, 0, 1, 0,        ! h  l  k   14
     1           1, 0, 0, 0, 0,-1, 0,-1, 0,        ! h -l -k   15
     1          -1, 0, 0, 0, 0,-1, 0, 1, 0/        !-h -l  k   16
      data mat4/
     1           1, 0, 0, 0, 1, 0, 0, 0, 1,        !   h     k     l   1
     1           0, 1, 0,-1,-1, 0, 0, 0, 1,        !   k  -h-k     l   2
     1          -1,-1, 0, 1, 0, 0, 0, 0, 1,        !-h-k     h     l   3
     1          -1, 0, 0, 0,-1, 0, 0, 0,-1,        !  -h    -k    -l   4
     1           0,-1, 0, 1, 1, 0, 0, 0,-1,        !  -k   h+k    -l   5
     1           1, 1, 0,-1, 0, 0, 0, 0,-1,        ! h+k    -h    -l   6
     1          -1, 0, 0, 1, 1, 0, 0, 0, 1,        !  -h   h+k     l   7
     1           1, 1, 0, 0,-1, 0, 0, 0, 1,        ! h+k    -k     l   8
     1           0,-1, 0,-1, 0, 0, 0, 0, 1,        !  -k    -h     l   9
     1           1, 0, 0,-1,-1, 0, 0, 0,-1,        !   h  -h-k    -l  10
     1          -1,-1, 0, 0, 1, 0, 0, 0,-1,        !-h-k     k    -l  11
     1           0, 1, 0, 1, 0, 0, 0, 0,-1,        !   k     h    -l  12
     1           0, 1, 0, 1, 0, 0, 0, 0, 1,        !   k     h     l  13
     1           1, 0, 0,-1,-1, 0, 0, 0, 1,        !   h  -h-k     l  14
     1          -1,-1, 0, 0, 1, 0, 0, 0, 1,        !-h-k     k     l  15
     1           0,-1, 0,-1, 0, 0, 0, 0,-1,        !  -k    -h    -l  16
     1          -1, 0, 0, 1, 1, 0, 0, 0,-1,        !  -h   h+k    -l  17
     1           1, 1, 0, 0,-1, 0, 0, 0,-1/        ! h+k    -k    -l  18
      data mat5/
     1           1, 1, 0,-1, 0, 0, 0, 0, 1,        ! h+k    -h     l   1
     1          -1, 0, 0, 0,-1, 0, 0, 0, 1,        !  -h    -k     l   2
     1           0,-1, 0, 1, 1, 0, 0, 0, 1,        !  -k   h+k     l   3
     1          -1,-1, 0, 1, 0, 0, 0, 0,-1,        !-h-k     h    -l   4
     1           1, 0, 0, 0, 1, 0, 0, 0,-1,        !   h     k    -l   5
     1           0, 1, 0,-1,-1, 0, 0, 0,-1,        !   k  -h-k    -l   6
     1           0, 1, 0, 1, 0, 0, 0, 0, 1,        !   k     h     l   7
     1           1, 0, 0,-1,-1, 0, 0, 0, 1,        !   h  -h-k     l   8
     1          -1,-1, 0, 0, 1, 0, 0, 0, 1,        !-h-k     k     l   9
     1           0,-1, 0,-1, 0, 0, 0, 0,-1,        !  -k    -h    -l  10
     1          -1, 0, 0, 1, 1, 0, 0, 0,-1,        !  -h   h+k    -l  11
     1           1, 1, 0, 0,-1, 0, 0, 0,-1/        ! h+k    -k    -l  12

      tol=1.0e-6
      rint=1.0
      isum=0
      fsum1=0.0
      fsum2=0.0
      kh=2**16
      kk=2**8
      kl=1
      k0=128
      korig=k0*(kh+kk+kl)
C
C ---- 1. ensure indices are in standard setting for each Laue group
C
      do i=1,nrefl
        im(1)=ih(i)
        im(2)=ik(i)
        im(3)=il(i)
        i1=vm(im,mat)

        if(imode .eq. 1) then
C -- -1  (triclinic) only Friedel pairs merged
          i2=vm(im,mat1(1,1,1))
          ihkl(i)=max0(i1,i2)

        elseif (imode .eq. 2) then
C -- 2/m (monoclinic)
          do k=1,3
            i2=vm(im,mat1(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 3) then
C -- mmm  (orthorhombic)
          do k=1,7
            i2=vm(im,mat1(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 4) then
C -- 4/m   (tetragonal)
          do k=1,11
            if(k.eq.1.or.k.eq.4.or.k.eq.5.or.k.ge.8) then
              i2=vm(im,mat1(1,1,k))
              if(i2 .gt. i1) i1=i2
            endif
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 5) then
C -- 4/mmmm (tetragonal)
          do k=1,15
            i2=vm(im,mat1(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 6) then
C -- -3 (trigonal rhomb-axes)
          i2=vm(im,mat1(1,1,1))
          if(i2 .gt. i1) i1=i2
          do k=1,13,4
            i2=vm(im,mat2(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 7) then
C --  -3m (trigonal rhomb-axes)
          i2=vm(im,mat1(1,1,1))
          if(i2 .gt. i1) i1=i2
          do k=14,15
            i2=vm(im,mat1(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=1,13,4
            i2=vm(im,mat2(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=2,14,4
            i2=vm(im,mat3(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 8) then
C --  -3 (trigonal hex-axes)
          do k=2,6
            i2=vm(im,mat4(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 9) then
C --  -3m1 (trigonal hex-axes)
          do k=2,12
            i2=vm(im,mat4(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 10) then
C --  -31m (trigonal hexagonal axes)
          do k=2,6
            i2=vm(im,mat4(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=13,18
            i2=vm(im,mat4(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 11) then
C --  6/m (hexagonal)
          do k=2,6
            i2=vm(im,mat4(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=1,6
            i2=vm(im,mat5(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 12) then
C --  6/mmm    (hexagonal)
          do k=2,12
            i2=vm(im,mat4(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=1,12
            i2=vm(im,mat5(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 13) then
C -- m-3 (cubic)
          do k=1,7
            i2=vm(im,mat1(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=1,16
            i2=vm(im,mat2(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1

        elseif (imode .eq. 14) then
C -- m-3m (cubic)
          do k=1,15
            i2=vm(im,mat1(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=1,16
            i2=vm(im,mat2(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          do k=1,16
            i2=vm(im,mat3(1,1,k))
            if(i2 .gt. i1) i1=i2
          enddo
          ihkl(i)=i1
        endif
      enddo
C
C ---- 2. sort reflection arrays
C
       do i=1,nrefl
         f_sorted(i)=f(i)
         sf_sorted(i)=sf(i)
         int(i)=0
       enddo
       call laue_sort(f_sorted,sf_sorted,ihkl,int,nrefl)
C      
C ---- 3. loop to sum equivalents
C
      icount=1
      isum1=0
      isave=ihkl(1)
      fsave1(1)=f_sorted(1)
      fsave2(1)=sf_sorted(1)

      do i=2,nrefl

        if(ihkl(i) .eq. isave) then

          icount=icount+1
          fsave1(icount)=f_sorted(i)
          fsave2(icount)=sf_sorted(i)
        elseif(ihkl(i) .ne. isave .and. icount. gt. 1) then
          if(imerge .eq. 2) then
            if(ifull .eq. 1) then
              jix=ihkl(i)
              call decode(jix,im)
              write(11,100) (im(k),k=1,3),(fsave1(k),k=1,icount)
            endif
            fmean=0.0
            do k=1,icount
              fmean=fmean+fsave1(k)
            enddo
            fmean=fmean/float(icount)
            do k=1,icount
              fsum1=fsum1+abs(fsave1(k)-fmean)
              fsum2=fsum2+fsave1(k)
            enddo
          elseif(imerge .eq. 1) then
            sw=0.
            swx=0.
            swx2=0.
            do k=1,icount
              x=fsave1(k)
              w=fsave2(k)
              w=1.0/(w*w)
              sw=sw+w
              swx=swx+w*x
              swx2=swx2+w*x*x
            enddo
            sigi=1.0/sw
            s=sw*swx2
            q=swx*swx
            fsum1=fsum1+real(icount)*(s-q)*sigi
            fsum2=fsum2+real(icount-1)*swx2
            if(ifull .eq. 1) then
              jix=ihkl(i)
              call decode(jix,im)
              write(11,100) (im(k),k=1,3),(fsave1(k),k=1,icount)
            endif
          endif
          isave=ihkl(i)
          isum=isum+icount
          isum1=isum1+1
          icount=1
          isave=ihkl(i)
          fsave1(1)=f_sorted(i)
          fsave2(1)=sf_sorted(i)
        elseif(ihkl(i) .ne. isave .and. icount. le. 1) then

          isave=ihkl(i)
          fsave1(1)=f_sorted(i)
          fsave2(1)=sf_sorted(i)
        endif
      enddo     

      if(fsum2 .gt. tol) then
        if(imerge .eq. 1) rint=sqrt(fsum1/fsum2)
        if(imerge .eq. 2) rint=(fsum1/fsum2)
      endif
100   format(3i4,2x,8f9.2/10f9.2/10f9.2/10f9.2/10f9.2)
      end
C
C----------------------------------------------------------------------C
C
      subroutine laue_sort(a,b,ix,iy,n)
C ---- sorts ix,iy,a,b with ix as the key
      real a(n),b(n)
      integer ix(n),iy(n)
      int=2
   10 int=2*int
      if(int.lt.n) go to 10
      int=min0(n,(3*int)/4-1)
   20 int=int/2
      ifin=n-int
      do 70 ii=1,ifin
      i=ii
      j=i+int
      if(ix(i).le.ix(j)) go to 70
      t=a(j)
      x=b(j)
      m=iy(j)
      l=ix(j)
   40 a(j)=a(i)
      b(j)=b(i)
      ix(j)=ix(i)
      iy(j)=iy(i)
      j=i
      i=i-int
      if(i.le.0) go to 60
      if(ix(i).gt.l) go to 40
   60 a(j)=t
      b(j)=x
      ix(j)=l
      iy(j)=m
   70 continue
      if(int.gt.1) go to 20
      end
C
C----------------------------------------------------------------------C
C
      subroutine decode(ihkl,im)
      common/laue_k/kh,kk,korig,k0
      integer im(3)
      jix=ihkl
      im(1)=jix/kh-k0
      jix=mod(jix,kh)
      im(2)=jix/kk-k0
      im(3)=mod(jix,kk)-k0
      end
C
C----------------------------------------------------------------------C
C
      integer function vm(ia,ib)
      common/laue_k/kh,kk,korig,k0
      integer ia(3),ic(3)
      integer ib(3,3)
      do i=1,3
        ic(i)=0
        do j=1,3
          ic(i)=ic(i)+ia(j)*ib(j,i)
        enddo
      enddo
      vm=ic(1)*kh+ic(2)*kk+ic(3)+korig
      end
C
C----------------------------------------------------------------------C
C
      integer function leng(string)
C ---- returns significant length of  'string'
      character*(*)string
      do 1 leng=len(string),0,-1
1        if (string(leng:leng) .ne. ' ') goto 2
2     end
C
C----------------------------------------------------------------------C
C
      subroutine upcase(string)
C ---- sets 'string' to upper case
      character*(*)string
      ilen=len(string)
      do 1 j=1,ilen
         if (lge(string(j:j),'a') .and. lle(string(j:j),'z'))
     1   string(j:j)=char(ichar(string(j:j))-ichar('a')+ichar('A'))
1     continue
      end
C
C----------------------------------------------------------------------C
C
      subroutine trim(string)
C ---- remove leading blanks from 'string' 
      character*(*)string
      integer i,ilen,iblank
      ilen=len(string)
      iblank=0
      do 1 i=1,ilen
         if (string(i:i) .ne. ' ') goto 2
1        if (string(i:i) .eq. ' ')iblank=iblank+1
2     if (iblank .ne. 0) then
         do 3 i=1,ilen-iblank
3           string(i:i)=string(i+iblank:i+iblank)
         string(ilen-iblank+1:ilen)=' '
      endif
      end
